@@ -16,6 +16,7 @@
 module Text.CSL.Eval
     ( evalLayout
     , evalSorting
+    , rtfParser
     , last'
     , module Text.CSL.Eval.Common
     , module Text.CSL.Eval.Output
@@ -44,7 +45,7 @@ import Text.ParserCombinators.Parsec hiding ( State (..) )
 evalLayout :: Layout   -> EvalMode -> Bool -> [Locale] -> [MacroMap]
            -> [Option] -> Reference -> [Output]
 evalLayout (Layout _ _ es) em b l m o r
-    = cleanOutput result
+    = cleanOutput evalOut
     where
       evalOut = case evalState job initSt of
                   [] -> if (isSorting $ em)
@@ -52,16 +53,6 @@ evalLayout (Layout _ _ es) em b l m o r
                         else [noOutputError]
                   x | title r == citeId cit ++ " not found!" -> [noBibDataError $ cit]
                     | otherwise                              -> x
-      addAff f x c = case f c of
-                       PlainText  []    -> []
-                       PlainText  p | x -> [Output (rtfParser emptyFormatting p) emptyFormatting, OSpace]
-                       PlainText  p     -> [OSpace, Output (rtfParser emptyFormatting p) emptyFormatting]
-                       PandocText []    -> []
-                       PandocText p | x -> [OPan p, OSpace]
-                       PandocText p     -> [OPan p]
-      result  = case em of
-                  EvalCite c -> addAff citePrefix True c ++ evalOut ++ addAff citeSuffix False c
-                  _          -> evalOut
       locale = case l of
                  [x] -> x
                  _   -> Locale [] [] [] [] []
@@ -71,7 +62,7 @@ evalLayout (Layout _ _ es) em b l m o r
                  EvalSorting c -> c
                  EvalBiblio  s -> emptyCite { citePosition = s }
       initSt = EvalState (mkRefMap r) (Env cit (localeTermMap locale) m
-                         (localeDate locale) o []) [] em b False [] False [] [] []
+                         (localeDate locale) o []) [] em b False [] [] False [] [] []
 
 evalSorting :: EvalMode -> [Locale] -> [MacroMap] -> [Option] -> [Sort] -> Reference -> [Sorting]
 evalSorting m l ms opts ss r
@@ -105,7 +96,7 @@ evalElement :: Element -> State EvalState [Output]
 evalElement el
     | Choose i ei e         <- el = evalIfThen i ei e
     | Macro    s   fm       <- el = return . appendOutput fm =<< evalElements =<< getMacro s
-    | Const    s   fm       <- el = return $ rtfParser fm $ unescape s
+    | Const    s   fm       <- el = return $ rtfParser fm s
     | Number   s f fm       <- el = formatNumber f fm =<< getStringVar s
     | Variable s f fm d     <- el = return . addDelim d =<< concatMapM (getVariable f fm) s
     | Group        fm d l   <- el = when' ((/=) [] <$> tryGroup l) $
@@ -115,7 +106,7 @@ evalElement el
     | Term     s f fm p     <- el = formatLabel f fm p s
     | Names    s n fm d sub <- el = modify (\st -> st { contNum = [] }) >>
                                     ifEmpty (evalNames False s n d)
-                                            (withNames el $ evalElements sub)
+                                            (withNames s el $ evalElements sub)
                                             (appendOutput fm)
     | Substitute (e:els)    <- el = ifEmpty (consuming $ evalElement e)
                                             (getFirst els) id
@@ -136,10 +127,12 @@ evalElement el
 
       ifEmpty p t e = p >>= \r -> if r == [] then t else return (e r)
 
-      withNames n f = modify (\s -> s { env = (env s)
-                                        {names = n : names (env s)}}) >> f >>= \r ->
-                      modify (\s -> s { env = (env s)
-                                        {names = tail $ names (env s)}}) >> return r
+      withNames e n f = modify (\s -> s { authSub = concat e
+                                        , env = (env s)
+                                          {names = n : names (env s)}}) >> f >>= \r ->
+                        modify (\s -> s { authSub = []
+                                        , env = (env s)
+                                          {names = tail $ names (env s)}}) >> return r
 
       getFirst        [] = return []
       getFirst    (x:xs) = whenElse ((/=) []  <$> evalElement x)
@@ -239,8 +232,10 @@ rtfParser fm s
       parser = parseText <|> parseMarkup
 
       parseText = do
+        let amper = string "&" >> notFollowedBy (char '#') >>
+                    return [OStr "&" emptyFormatting]
         x  <- many $ noneOf "<'\"`“‘&"
-        xs <- parseQuotes <|> parseMarkup
+        xs <- parseQuotes <|> parseMarkup <|> amper
         r  <- manyTill anyChar eof
         return (OStr x emptyFormatting : xs ++
                 [Output (rtfParser emptyFormatting r) emptyFormatting])
@@ -355,9 +350,12 @@ minimalRange sa sb
     where
       (a,b) = expandedRange sa sb
       res   = if length a == length b
-              then second (filter (/= '+')) $ unzip $
-                   zipWith (\x y -> if x == y then (x,'+') else (x,y)) a b
+              then second (filter (/= '+')) $ unzip $ doit a b
               else (a,b)
+      doit (x:xs) (y:ys) = if x == y
+                           then (x,'+') : doit xs ys
+                           else zip (x:xs) (y:ys)
+      doit _      _      = []
 
 chicagoRange :: String -> String -> (String, String)
 chicagoRange sa sb
@@ -366,7 +364,7 @@ chicagoRange sa sb
           | '0':'0':_ <- sa' -> expandedRange sa sb
           | _  :'0':_ <- sa' -> minimalRange  sa sb
           | _  :a2:as <- sa'
-          , b1:b2:bs  <- sb'
+          , b1 :b2:bs <- sb'
           , comp as bs       -> if a2 == b2
                                 then (sa, [b2,b1])
                                 else minimalRange sa sb
@@ -382,12 +380,6 @@ chicagoRange sa sb
         comp a b = let b' = takeWhile isDigit b
                    in take (length b') a == b'
 
-unescape :: String -> String
-unescape [] = []
-unescape ('&':'l':'t':';':xs) = "<" ++ unescape xs
-unescape ('&':'g':'t':';':xs) = ">" ++ unescape xs
-unescape (x              :xs) = x    : unescape xs
-
 last' :: [a] -> [a]
 last' = foldl (\_ x -> [x]) []
 
