@@ -44,11 +44,11 @@ xpMods = xpIElem "mods" xpReference
 xpReference :: PU Reference
 xpReference
     = xpWrap ( \ ( ref
-                , (ck,ty,ti)
+                , (ck,ty,ti,i,d)
                 ,((au,ed,tr),(re,it,pu'),(co,ce))
                 ,((di',pg,vl,is),(nu,sc,ch))
                 , (di,pu,pp)
-                , (ac,uri)
+                , ((ac,uri),no)
                  ) ->
                ref { refId            = ck `betterThen` take 10 (concat $ words ti)
                    , refType          = if refType ref /= NoType then refType ref else ty
@@ -63,7 +63,10 @@ xpReference
                    , publisherPlace   = pp `betterThen` publisherPlace   ref
                    , containerAuthor  = containerAuthor  ref
                    , url              = uri
+                   , note             = no
                    , accessed         = ac
+                   , isbn             = i
+                   , doi              = d
                    , issued           = issued        ref `betterThen` di `betterThen` di'
                    , page             = page          ref `betterThen` pg
                    , volume           = volume        ref `betterThen` vl
@@ -76,18 +79,19 @@ xpReference
                                            `betterThen` (foldr (<+>) [] . map show $ pu')
                    }
              , \r -> (  r
-                     , (refId     r, refType          r, title          r)
+                     , (refId     r, refType          r, title          r, isbn r, doi r)
                      ,((author    r, editor           r, translator     r)
                       ,(recipient r, interviewer      r, emptyAgents     )
                       ,(composer  r, collectionEditor r))
-                     ,((issued  r, page  r, volume r, issue r)
-                      ,(number  r, section   r, chapterNumber r))
+                     ,((issued    r, page  r, volume r, issue r)
+                      ,(number    r, section   r, chapterNumber r))
                      , (issued    r, emptyAgents, publisherPlace r)
-                     , (accessed  r, url              r)
+                     ,((accessed  r, url r), note r)
                      )) $
       xp6Tuple (xpDefault emptyReference xpRelatedItem)
-               (xpTriple  xpCiteKey xpRefType xpTitle )
-                xpAgents xpPart xpOrigin xpUrl
+               (xp5Tuple  xpCiteKey xpRefType xpTitle xpIsbn xpDoi)
+                xpAgents xpPart xpOrigin
+                (xpPair xpUrl xpNote)
 
 xpCiteKey :: PU String
 xpCiteKey
@@ -221,10 +225,11 @@ parseName gn fn
     | otherwise                 = parse gn  ""         False
     where
       parse g s b = Agent (getGiven g) (getDrop g) (getNonDrop fn) (getFamily fn) s [] b
-      getDrop     = unwords . filter (and . map isLower)
-      getGiven    = filter (not . and . map isLower)
-      getNonDrop  = getDrop . words
-      getFamily   = unwords . getGiven . words
+      setInit   s = if length s == 1 then s ++ "." else s
+      getDrop     = unwords     . reverse . takeWhile (and . map isLower) . reverse
+      getGiven    = map setInit . reverse . dropWhile (and . map isLower) . reverse
+      getNonDrop  = unwords . takeWhile (and . map isLower) . words
+      getFamily   = unwords . dropWhile (and . map isLower) . words
 
 xpNameData :: PU ([String],String)
 xpNameData
@@ -295,6 +300,19 @@ xpUrl
               xpDefault [] $ xpAttr "dateLastAccessed" xpText)
              (xpDefault [] $ xpElem "url"              xpText)
 
+xpIsbn :: PU String
+xpIsbn = xpDefault [] $ xpIdentifier "isbn"
+
+xpDoi :: PU String
+xpDoi = xpDefault [] $ xpIdentifier "doi"
+
+xpIdentifier :: String -> PU String
+xpIdentifier i
+    = xpIElem "identifier" $ xpAddFixedAttr "type" i xpText
+
+xpNote :: PU (String)
+xpNote = xpDefault [] $ xpIElem "note" xpText
+
 readDate :: String -> [RefDate]
 readDate s = if takeWhile isDigit s /= []
              then return $ RefDate (takeWhile isDigit s) [] [] [] [] []
