@@ -1,4 +1,5 @@
 {-# LANGUAGE PatternGuards, CPP #-}
+{-# LANGUAGE TemplateHaskell #-}
 {-# OPTIONS_GHC -fno-warn-orphans #-}
 -----------------------------------------------------------------------------
 -- |
@@ -16,12 +17,20 @@
 
 module Text.CSL.Parser where
 
-import Paths_citeproc_hs ( getDataFileName )
 import Text.CSL.Reference ( readNum )
 import Text.CSL.Style
 import Text.CSL.Pickle
+#ifdef EMBED_DATA_FILES
+import Data.FileEmbed
+import qualified Data.ByteString as BS
+import Data.ByteString.UTF8 ( toString )
+#else
+import Paths_citeproc_hs ( getDataFileName )
+import System.Directory  ( doesFileExist   )
+#endif
 
 import Data.Char        ( isUpper, toUpper, toLower )
+import Data.Maybe       ( catMaybes                 )
 
 #ifdef USE_NETWORK
 import Network.HTTP ( getResponseBody, mkRequest, RequestMethod(..) )
@@ -49,14 +58,37 @@ readCSLFile src = do
   l <- readLocaleFile (styleDefaultLocale s)
   return s { styleLocale = mergeLocales (styleDefaultLocale s) l (styleLocale s)}
 
+
+#ifdef EMBED_DATA_FILES
+localeFiles :: [(FilePath, BS.ByteString)]
+localeFiles = $(embedDir "locales/")
+#endif
+
 readLocaleFile :: String -> IO Locale
 readLocaleFile s = do
+#ifdef EMBED_DATA_FILES
+  f <- case s of
+         x | length x == 2 -> let fn = ("locales-" ++
+                                        maybe (error $ x ++ " doesn't seem a valid locale")
+                                        id (lookup x langBase) ++ ".xml")
+                              in case lookup fn localeFiles of
+                                   Just x' -> return x'
+                                   _       -> error "could not load the locale file"
+            | otherwise     -> case lookup ("locales-" ++ take 5 x ++ ".xml") localeFiles of
+                                 Just x' -> return x'
+                                 _       -> error "could not load the locale file"
+  return $ readXmlString xpLocale (toString f)
+#else
   f <- case s of
          x | length x == 2 -> getDataFileName ("locales/locales-" ++
                                                maybe (error $ x ++ " doesn't seem a valid locale")
                                                id (lookup x langBase) ++ ".xml")
            | otherwise     -> getDataFileName ("locales/locales-" ++ take 5 x ++ ".xml")
-  readXmlFile xpLocale f
+  b <- doesFileExist f
+  if b
+     then readXmlFile xpLocale f
+     else readLocaleFile $ take 2 s
+#endif
 
 instance XmlPickler Layout where
     xpickle = xpWrap (uncurry3 Layout, \(Layout f d e) -> (f,d,e)) $
@@ -290,27 +322,32 @@ xpNamesOpt = xpOpt "names-delimiter"
 xpNameFormat :: PU [Option]
 xpNameFormat
     = xpWrap (\(a,b,c,d,e) ->
-                  filter ((/=) [] . snd) $ [("and",a)
-                                           ,("delimiter-precedes-last",b)
-                                           ,("sort-separator",c)
-                                           ,("initialize-with",d)
-                                           ,("name-as-sort-order",e)], const ([],[],[],[],[])) $
-      xp5Tuple (xpAttrText' "and")
-               (xpAttrText' "delimiter-precedes-last")
-               (xpAttrText' "sort-separator")
-               (xpAttrText' "initialize-with")
-               (xpAttrText' "name-as-sort-order")
+                  catMaybes [ checkOpt "and"                     a
+                            , checkOpt "delimiter-precedes-last" b
+                            , checkOpt "sort-separator"          c
+                            , checkOpt "initialize-with"         d
+                            , checkOpt "name-as-sort-order"      e
+                            ] , const (Nothing, Nothing, Nothing, Nothing, Nothing)) $
+      xp5Tuple (getOpt "and")
+               (getOpt "delimiter-precedes-last")
+               (getOpt "sort-separator")
+               (getOpt "initialize-with")
+               (getOpt "name-as-sort-order")
 
+    where
+      getOpt n = xpOption $ xpAttr n xpText
+      checkOpt _ Nothing  = Nothing
+      checkOpt n (Just s) = Just (n,s)
 
 xpNameAttrs :: PU NameAttrs
 xpNameAttrs
     = xpWrap (\((a,b,c,d,e),(f,g)) ->
-                  filter ((/=) [] . snd) $ [("et-al-min",a)
-                                           ,("et-al-use-first",b)
-                                           ,("et-al-subsequent-min",c)
-                                           ,("et-al-subsequent-use-first",d)
-                                           ,("et-al-use-last",e)
-                                           ,("delimiter-precedes-et-al",f)] ++ g
+                  filter ((/=) [] . snd) [("et-al-min",a)
+                                         ,("et-al-use-first",b)
+                                         ,("et-al-subsequent-min",c)
+                                         ,("et-al-subsequent-use-first",d)
+                                         ,("et-al-use-last",e)
+                                         ,("delimiter-precedes-et-al",f)] ++ g
              , const (([],[],[],[],[]),([],[]))) $
       xpPair (xp5Tuple (xpAttrText' "et-al-min")
                        (xpAttrText' "et-al-use-first")
@@ -419,9 +456,10 @@ xpNames
     = xpWrap ( \((a,n,fm),d,sb)     -> Names (words a) n fm d sb
              , \(Names a n fm d sb) -> ((unwords a,n,fm),d,sb)) $
       xpElem "names" $ xpTriple names xpDelimiter xpickle
-    where names   = xpTriple (xpAttrText "variable") xpName xpickle
-          xpName  = xpChoice xpZero xpickle check
-          check l = if any isName l then xpLift l else xpZero
+    where names    = xpTriple (xpAttrText "variable") xpName xpickle
+          xpName   = xpChoice xpZero xpickle check
+          check [] = xpLift [Name NotSet emptyFormatting [] [] []]
+          check  l = if any isName l then xpLift l else xpZero
 
 xpShortNames :: PU Element
 xpShortNames
