@@ -1,5 +1,5 @@
 import System.Environment
-import Text.CSL.Input.Json
+import Text.CSL.Test
 
 -- a dir with a final slash
 testDir = "citeproc-test/processor-tests/machines/"
